
# tail-rank-power.R
# Copyright, Kevin R. Coombes, 2004.

require('methods')

#########################################################################
# A power estimate can be obtained as follows.  First, let X ~ Binom(N,p)
# denote a binomial random variable. Under the null hypotheis that cancer
# is not different from normal, we can choose p = 1 - psi for the expected
# proportion of successes in a test of whether the value exceeds the
# psi-th quantile.  Now let
#	alpha = P(X > x,| N, p)
# for one such binomial measurement.  When we make G independent binomial
# measurements, we take
#	conf = P(all G of the X's <= x | N, p).

# (In our paper on the tail-rank statistic, we write everything in terms
# of gamma = 1 - conf.) Then we have
#	conf = P(X <= x | N, p)^G = (1 - alpha)^G.
# Using a Bonferroni-like approximation, we can take
#	conf ~= 1 - # alpha*G.
# Solving for alpha, we find that
#	alpha ~= (1-conf)/G.
# So, the cutoff that ensures that in multiple experiments, each looking
# at G genes in N samples, we have confidence level conf (or signficance
# level gamma = 1 - conf) of no false positives is given by the following
# function. The final point to note is that the quantiles are also defined
# in terms of q = 1 - alpha, so there are lots of disfiguring "1's" in the
# implementation.
tail.rank.cutoff <- function(G, N, psi, conf, method='approx') {
  known.methods <- c('approx', 'exact')
  which <- charmatch(method, known.methods)
  if (is.na(which)) {
    stop(paste('unknown method:', method))
  }
  if (which == 1) {
    q <- 1 - (1-conf)/G
  } else {
    q <- conf^(1/G)
  }
  qbinom(q, N, 1-psi)
}

# Now we again set M to be the significance cutoff using the procedure
# detailed above.  A gene with sensitivity phi gets detected if the
# observed number of cases above the threshold is greater than or equal
# to M. So, we have a function that implements formula (1.3) of the paper
# on the tail-rank test.
tail.rank.power <- function(G, N, psi, phi, conf=0.95) {
  M <- tail.rank.cutoff(G, N, psi, conf)
  1-pbinom(M, N, phi)
}

#########################################################################
# Now we introduce a class to generate power tables.

setClass('bmpt',
         representation(G='numeric',
                        psi='numeric',
                        conf='numeric',
                        power='data.frame')
         )


bmpt <- function(G, psi, conf, power) {
  new('bmpt', G=G, psi=psi, conf=conf, power=power)
}

setMethod('summary', 'bmpt', function(object, ...) {object})

setMethod('print', 'bmpt', function(x, ...) {
  cat(paste('G = ', x@G, ', psi = ', x@psi,
            ', conf = ', x@conf, '\nPower:\n', sep=''))
  print(x@power)
})

biomarker.power.table <-
  function(G,
           N=seq(25, 250, by=25),
           psi=c(0.95, 0.99),
           conf=0.99,
           phi=seq(0.10, 0.50, by=0.05)) {
  val <- list()
  i <- 1
  for (g in G) {
    for (p in psi) {
      for (gam in conf) {
	pows <- matrix(0, ncol=length(phi), nrow=length(N))
	for (ph in 1:length(phi)) {
          pows[, ph] <-  tail.rank.power(g, N, p, phi[ph], gam)
	}
	pows <- data.frame(pows)
	dimnames(pows) <- list(as.character(N), as.character(round(100*phi)))
        extra <- bmpt(g, p, gam, pows)
        val[[i]] <-  extra
        i <- i + 1
      }
    }
  }
  val
}

