
# tail-rank-test.R
# Copyright: Kevin R. Coombes, 2004.

require('methods')

# Implements the tail-rank test described in our paper.

############################
# We start with a class to hold and display the results.

setClass("tail.rank.test",
         representation(statistic='numeric',	# nonnegative integer vector
                        direction='character',	# up, down, or two-sided
                        N1='numeric',		# number of healthy
                        N2='numeric',		# number of cancer
                        tau='numeric or NULL',  # upper thresholds
                        rho='numeric or NULL',  # lower thresholds
                        specificity='numeric',	# between 0 and 1
                        tolerance='numeric',	# between 0 and 1
                        confidence='numeric',	# between 0 and 1
                        cutoff='numeric'	# integer: max value under null
                        )
         )

setMethod('summary', 'tail.rank.test', function(object, ...) {
  cat(paste('A tail-rank test object in the',
            object@direction,'direction.\n'))
  cat(paste('Specificity:', object@specificity,
            'computed with tolerance', object@tolerance, '\n'))
  cat(paste('Significance cutoff:', object@cutoff,
            'based on a family-wise error rate less than',
            1-object@confidence, '\n'))
  cat(paste('There are', sum(as.logical(object)),
            'tail-rank statistics that exceed the cutoff\n'))
})

setMethod('hist', 'tail.rank.test',
          function(x,  overlay=FALSE,
                   xlab='tail-rank statistic', main='', ...) {
            if(overlay) {
              xx <- 0:length(x@statistic)
              qq <- 1 - pnorm(tol.bound(x@specificity, x@tolerance, x@N1))
              dd <- dbinom(xx, x@N2, qq)
              hist(x@statistic, xlab=xlab, main=main,
                   prob=TRUE, ...)
              lines(xx, dd, col='red')
            } else {
              hist(x@statistic, xlab=xlab, main=main, ...)
            }
            # also add a tick mark at the significance cutoff
            rug(x@cutoff)
            invisible(x)
          })

setMethod('as.logical', 'tail.rank.test', function(x, ...) {
  x@statistic > x@cutoff
})

if (!isGeneric('getStatistic'))
  setGeneric('getStatistic', function(object, ...) {
    standardGeneric('getStatistic')})

setMethod('getStatistic', 'tail.rank.test',
          function(object, ...) {object@statistic})

############################
# Here is the constructor that actually performs the test
#
# We compute the tail rank statistic for each gene (as rows of
# the data matrix). The data is split into two groups; the
# first group is used to estimate a tolerance bound (defaults
# to 90%) on a specific quantile (defaults to 95%) of the
# distribution of each gene. The tail-rank statistic is the
# number of samples in the second group outside the bound.
# The test can be applied in the "up", "down", or "two-sided"
# direction, depending on the kinds of markers being sought.
# Also computes the cutoff for significance based on a confidence
# level that is "1 - FWER" for a desired family-wise error rate.
tail.rank.test <-
  function(data,
           split,
           specificity=0.95,
           tolerance=0.90,
           confidence=0.95,
           direction='up') {

    # check consistency of the arguments
    # start with the direction of the test
    directions <- c('up', 'down', 'two-sided')
    dir <- pmatch(direction, directions)
    if(is.na(dir))
      stop(paste(direction, 'is not a valid direction for comparison'))
    if(dir == -1)
      stop(paste(direction, 'is ambiguous'))

      # now get split as a logical vector
    if(is.factor(split))
      split <- split == levels(split)[[1]]
    split <- as.logical(split)
    if(sum(split)==0 | sum(!split)==0)
      stop('one of the groups is empty!')

    # the data should be a matrix, with number of columns = length of split
    data <- as.matrix(data)
    if(ncol(data) != length(split))
      stop(paste('The number of data columns must equal the',
                 'length of the splitting vector'))

    # make sure specificity, tolerance, and confidence are in the correct range
    if (specificity > 1 | specificity < 0)
      stop('specificity must be between 0 and 1')
    if (tolerance > 1 | tolerance < 0)
      stop('tolerance must be between 0 and 1')
    if (confidence > 1 | confidence < 0)
      stop('confidence must be between 0 and 1')

    # we're finally ready to start
    healthy.n <- sum(split)
    cancer.n  <- sum(!split)
    healthy.mean <- matrix.mean(data[,split])
    cancer.mean  <- matrix.mean(data[,!split])
    healthy.sd <- sqrt(matrix.var(data[,split], healthy.mean))
    cancer.sd  <- sqrt(matrix.var(data[,!split], cancer.mean))
    tolerance.factor <- tol.bound(specificity, tolerance, healthy.n)

    # Now we actually compute the tail rank statistics
    tumors <- as.data.frame(data[,!split]) # i really hate this
    # you need to use a data.frame in order to get the comparison
    # operators to work correctly
    tau <- NULL
    rho <-  NULL
    if(dir == 1) { # up
      tau <- healthy.mean + healthy.sd*tolerance.factor
      trs <- as.matrix(tumors > tau) %*% rep(1, ncol(tumors))
      cutoff <- tail.rank.cutoff(nrow(data), sum(!split), specificity,
                                 confidence)
    } else if (dir == 2) { # down
      rho <- healthy.mean - healthy.sd*tolerance.factor
      trs <- as.matrix(tumors < rho) %*% rep(1, ncol(tumors))
      cutoff <- tail.rank.cutoff(nrow(data), sum(!split), specificity,
                                 confidence)
    } else { # two-sided
      tau <- healthy.mean + healthy.sd*tolerance.factor
      ups <- as.matrix(tumors > tau) %*% rep(1, ncol(tumors))
      rho <- healthy.mean - healthy.sd*tolerance.factor
      downs <- as.matrix(tumors < rho) %*% rep(1, ncol(tumors))
      trs <- ((ups + downs) + abs(ups-downs))/2	# maximum of one-sided
      cutoff <- tail.rank.cutoff(2*nrow(data), sum(!split), specificity,
                                 confidence)
    }

    trs <- as.vector(trs)
    # let's build a reasonable object for the return journey
    new("tail.rank.test",
        statistic=trs,
        direction=directions[dir],
        N1=sum(split),
        N2=sum(!split),
        tau=as.vector(tau), rho=as.vector(rho),
        specificity=specificity,
        tolerance=tolerance,
        confidence=confidence,
        cutoff=cutoff
        )
}

